
import { promises } from "fs";
import * as d3 from "d3-color";
import * as d3cd from "d3-color-difference";


class ColourNamer {
	constructor() {
		this.colmap = new Map();
	}

	async load_colours(colourfile) {
		const colourBuff = await promises.readFile(colourfile);
		const colours = colourBuff.toString();
		const seen = new Set(); 
		colours.split(/\n/).map((l) => {
			const m = l.match(/(\d+)\s+(\d+)\s+(\d+)\s+(.*)/);
			if( m ) {
				const sig = `${m[1]},${m[2]},${m[3]}`;
				if( !seen.has(sig) ) {
					this.colmap.set(m[4], d3.rgb(m[1], m[2], m[3]));
					seen.add(sig);
				}
			}
		});
	}

	colour_to_text(d3color) {
		const diffs = []
		for( const [ name, colour] of this.colmap) {
			diffs.push({name: name, dist: d3cd.differenceCie76(colour, d3color)});
		}
		diffs.sort((a, b) => a.dist - b.dist);
		return diffs[0].name;
	}
}


export { ColourNamer}