// calculate tiles


const RADIUS_OPTS = [ 
	"const",
	"right",
	"left",
	"up",
	"down",
	"right-up",
	"right-down",
	"left-up",
	"left-down",
	"circle-in",
	"circle-out",
	"hyper-in",
	"hyper-out",
	"grid",
	"noise",
];

const RADIUS_DESC = {
	"const": "of constant size",
	"right": "getting larger towards the right",
	"left": "getting larger towards the left",
	"up": "getting larger towards the top",
	"down": "getting larger towards the bottom",
	"right-up": "getting larger towards the upper right",
	"right-down": "getting larger towards the lower right",
	"left-up": "getting larger towards the upper left",
	"left-down": "getting larger towards the lower left",
	"circle-in": "forming a circle at the centre",
	"circle-out": "leaving a circular gap at the centre",
	"hyper-in": "forming a starlike pattern at the centre",
	"hyper-out": "leaving a starlike gap in the centre",
	"noise": "of random sizes",
	"grid": "forming a grid pattern",
}

function distance(dx, dy) {
	return Math.sqrt(dx ** 2 + dy ** 2);
}


function int_range(v1, v2) {
	const vs = [v1, v2];
	vs.sort((a, b) => a - b);
	const low = Math.floor(vs[0] - 1);
	const high = Math.ceil(vs[1] + 1);
	return [...Array(high - low + 1).keys()].map((i) => i + low);
}

function randint(min, max) {
	return min + Math.floor(Math.random() * (max + 1));
}

class DotMaker {
	constructor(width, height) {
		this.width = width;
		this.height = height;
		this.wh = 0.5 * (width + height);
		this.cx = 0.5 * width;
		this.cy = 0.5 * height;
	}

	dots(m, n, clip) {
		if( m - n === 0 ) {
			return [];
		}
		const ps = [];
		const is = int_range(-this.width, this.height / m)
		is.map((i) => {
			const js = int_range(m * i + (m - n) * this.width, m * i)
			js.map((j) => {
				const x = (j - m * i) / (m - n);
				const y = m * (x + i);
				if( !clip || (x > 0 && y > 0 && x < this.width && y < this.height) ) {
					ps.push({i:i, j:j, x:x, y:y});
				}
			});
		});
		return ps;
	}
}


function radius_func(func) {
  	switch (func) {
    	case "const":
      		return (d, r, w, h) => r;
    	case "right":
      		return (d, r, w, h) => r * d.x / w;
    	case "left":
      		return (d, r, w, h) => r * (w - d.x) / w;
    	case "down":
      		return (d, r, w, h) => r * d.y / h;
    	case "up":
      		return (d, r, w, h) => r * (h - d.y) / h;
      	case "right-up":
      		return (d, r, w, h) => r * (d.x + h - d.y) / (w + h);
      	case "left-up":
      		return (d, r, w, h) => r * (w - d.x + h - d.y) / (w + h);
      	case "right-down":
      		return (d, r, w, h) => r * (d.x + d.y) / (w + h);
      	case "left-down":
      		return (d, r, w, h) => r * (w - d.x + d.y) / (w + h);
      	case "circle-out":
      		return (d, r, w, h) => 4 * r * distance((d.x - w / 2), (d.y - h / 2)) / (w + h);
      	case "circle-in":
      		return (d, r, w, h) => 4 * r * (0.5 * (w + h) - distance((d.x - w / 2), (d.y - h / 2))) / (w + h);
      	case "hyper-in":
      		return (d, r, w, h) => 2 * r * (1 - Math.abs((d.x - w / 2) * (d.y - h / 2)) / (w + h));
      	case "hyper-out":
      		return (d, r, w, h) => 2 * r * Math.abs((d.x - w / 2) * (d.y - h / 2)) / (w + h);
      	case "noise":
      		return (d, r, w, h) => r * Math.random();
      	case "grid":
      		const xk = Math.PI * (2 * randint(1, 10) - 1);
      		const yk = Math.PI * (2 * randint(1, 10) - 1);
      		return (d, r, w, h) => r * (0.5 + 0.5 * (Math.sin(xk * d.x / w) + Math.sin(yk * d.y / h)));
    	default:
      		return (d, r, w, h) => r;
  	}
}

export { RADIUS_OPTS, RADIUS_DESC, DotMaker, radius_func };


